
{*******************************************************}
{                                                       }
{         Delphi VCL Extensions (ho)                    }
{            for  FastReport v2.2                       }
{       Text export filter (RTF Filter)                 }
{                                                       }
{     Copyright (c) 1996 - 1999 by Bartels Alex         }
{             JSIC "Energogarant"                       }
{             alex@msk-garant.ru                        }
{*******************************************************}

unit Fr_ertf;

interface

{$I FR_Vers.inc}

uses
  SysUtils, Windows, Messages, Classes, Graphics, FR_Class, FR_Utils, dialogs,
  fr_ertfClass;

type
  TfrRTFExportFilter = class(TfrExportFilter)
  private
    CurPage: TfrWordList;
    FirstPage: boolean;
    procedure WritePage;
  public
    constructor Create(AStream:TStream); override;
    destructor Destroy; override;
    procedure OnBeginDoc; override;
    procedure OnEndDoc; override;

    procedure OnEndPage; override;
    procedure OnBeginPage; override;
    procedure OnText(x,y:Integer; const text:String; View:TfrView); override;
  end;


implementation

const
  cNextLine = 4;    {   .      }

  {  . :
    3.97845 = 14/2.54*96/133
       : 14 -   133   MS Word  (MS San Serif  Size=12)
       : 2.54 -  -> 
       : 96 -   /    }
  SpaceLength12 =3.97845;    {    MS San Serif  Size=12 ( )}
  SpaceLength10 =2.98846365; {    MS San Serif  Size=10 ( )}

 {  .  min := abs(widthSpace - (X*SpaceLength10+ Y*SpaceLength12))
    X,Y  }
procedure Calc1012(widthSpace:Integer; var im10, im12:integer);
var
 ic10,ic12, i, j:integer;
 mValue, mMax, cr:extended;
begin
 ic10:=Round(widthSpace/SpaceLength10);
 ic12:=Round(widthSpace/SpaceLength12);
 mValue:=1000000; im10:=0; im12:=0;
 i:=0;
 while i<=ic12 do
  begin
   mMax:=1000000;
   j:=0;
   while j<=ic10 do
    begin
     cR:=abs(widthSpace-(j*SpaceLength10+i*SpaceLength12));
     if cR<mValue then
      begin
       mValue:=cR;
       im12:=i; im10:=j;
       mMax:=cR;
      end;
     if cR>mMax then j:=ic10; // 
     j:=j+1;
    end;
   i:=i+1;
  end;
end;

  {     widthSpace }
function DupSpace(widthSpace:Integer): string;
var
  im10, im12 : Integer;

  function dupSpace2(count:integer):string;
  var
   i:integer;
  begin
   result:='';
   for i:=1 to count do result:=result+'\''20';
  end;

begin
  result := '';
  if widthSpace=0 then exit;
  Calc1012(widthSpace,im10,im12);
  result:=dupSpace2(im12);
  result:='\f3\fs24'+result;
  if im10>0 then
     begin
      result:=result+'\f3\fs20';
      result:=result+dupSpace2(im10);
     end;
end;

constructor TfrRTFExportFilter.Create(AStream:TStream);
begin
  inherited Create(AStream);
  CurPage:= TfrWordList.Create;
  FirstPage:=True;
end;

destructor TfrRTFExportFilter.Destroy;
begin
  CurPage.Free;
  inherited Destroy;
end;

procedure TfrRTFExportFilter.OnBeginDoc;
begin
 {    } 
 CreateHeader(Stream);
end;

procedure TfrRTFExportFilter.OnEndDoc;
begin
 {   }
 WriteString(Stream,' } ');
end;

procedure TfrRTFExportFilter.OnEndPage;
begin
 {     }
  WritePage;
end;

procedure TfrRTFExportFilter.OnBeginPage;
begin
 {   }
 CurPage.Clear;
 if not FirstPage then
    WriteString(Stream,' \page ');
 FirstPage:=False;   
end;

procedure TfrRTFExportFilter.OnText(x,y:Integer; const text:String; View:TfrView);
begin
 if text<>'' then
    CurPage.AddWord(x,y,View.Canvas.Font,text,view.Canvas.TextWidth(text));
end;

 {     }
procedure TfrRTFExportFilter.WritePage;
var
 i:integer;
 s,sl:string;
 b:byte;
 prevY,y1,x1,tx:integer;

 styleWord, styleEndWord :string;
 FS: TFontStyles;
 FName:string;

 {   Fname,   ,     Ps }
 procedure intSetFont(inFname,Ps:string);
 begin
  if Pos(inFname,FName)<>0 then
    begin
     styleWord:=styleWord+Ps;
     styleEndWord:=styleEndWord+'\f0';
    end;
 end;

begin
 prevY:=0; tx:=0; sl:='';
 if CurPage.Count>0 then
  begin

   CurPage.SortData;
   for i:=0 to CurPage.Count-1 do
    begin
     y1:=Tfr_word(CurPage.Items[i]^).y;
     if Abs(PrevY-Y1)>cNextLine then
       begin
        PrevY:=y1;
        WriteString(stream,sl);
        sl:='';
        WriteString(stream,' \par ');
        b := 13;
        Stream.Write(b,1);
        tx:=0;
       end;

      {     } 
      x1:=Tfr_word(CurPage.Items[i]^).x-tx;

      s:=Tfr_word(CurPage.Items[i]^).Value;
      styleWord:='\fs'+IntToStr(Tfr_word(CurPage.Items[i]^).FontSize*2);
      styleEndWord:='';
        {   }
      FS:=Tfr_word(CurPage.Items[i]^).FontStyle;
      if fsBold in FS then
        begin
         styleWord:=styleWord+'\b';
         styleEndWord:=styleEndWord+'\b0';
        end;
      if fsItalic in FS then
        begin
         styleWord:=styleWord+'\i';
         styleEndWord:=styleEndWord+'\i0';
        end;
      if fsUnderline in FS then
        begin
         styleWord:=styleWord+'\ul';
         styleEndWord:=styleEndWord+'\ul0';
        end;
      if fsStrikeOut in FS then
        begin
         styleWord:=styleWord+'\strike';
         styleEndWord:=styleEndWord+'\strike0';
        end;
      {   }
      FName:=Tfr_word(CurPage.Items[i]^).FontName;
      intSetFont('Symbol','\f1');
      intSetFont('Arial','\f2');
      intSetFont('Sans','\f3');
      intSetFont('Courier','\f4');
      intSetFont('Times','\f5');

      s:=styleWord+' '+s+styleEndWord;
      sl:=sl+DupSpace(x1)+s;
      tx:=Tfr_word(CurPage.Items[i]^).width+Tfr_word(CurPage.Items[i]^).x;
      
    end;
   WriteString(stream,sl);
  end;
end;

initialization
  frRegisterExportFilter(TfrRTFExportFilter,
    frLoadStr('SRTFFile') + ' (*.rtf)','*.rtf');

end.
